//-----------------------------------------------------------------------------
// XAPP715_testbench - Module
//-----------------------------------------------------------------------------
//**************************************************************************
// 
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION \"AS IS\"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
//
//**************************************************************************
//-----------------------------------------------------------------------------
// Filename:        XAPP715_testbench.v
//-----------------------------------------------------------------------------
// Description:
//      Top level testbench for 16-bit Multiple Error Detection and Correction.
//		  Further details can be found in XAPP715
//-----------------------------------------------------------------------------
//Structure:
//    XAPP715_testbench.v
//     |--XAPP715_top.v
//        |--code_gen_s2.v
//        |--code_gen_s3.v
//        |--MLD_s1.v
//           |--MLD_adder_s1.v
//        |--MLD_s2.v
//           |--MLD_adder_s2.v
//        |--MLD_s3.v
//        |--ocs_gen_s1.v
//        |--ocs_gen_s2.v
//        |--encoder.v
//
//-----------------------------------------------------------------------------
// Owners:          Simon Tam 
// Revision 1.1
//
//     Modification History:                                                   
//     Date     Init          Description                                
//   --------  ------ ---------------------------------------------------------    
//   11/9/2004    STAM	    ver1.0 Initial Release 
//   3/31/2005    STAM      ver1.1 removed FDDRCPE in clkrst.v
//-----------------------------------------------------------------------------
`timescale 1ns/100ps

module XAPP715_testbench ();

   // Component Declaration
   parameter tclk = 10; 
   reg clk; 
   wire clk2x; 
   reg reset; 
   wire load; 
   reg[31:0] count; 
   wire [31:0] encin;
   wire[31:0] decin; 
   wire[31:0] encout; 
   wire[15:0] decout; 
   reg[31:0] encout_1; 
   reg[31:0] encout_2; 
   reg[31:0] encout_3; 
   reg[31:0] encout_4, encout_5, encout_6, encout_7, encout_8, encout_9; 
   wire[6:0] parity_out; 
   reg[2:0] force_error; 
   wire[1:0] error; 

   // Component Instantiation
   XAPP715_top uut (
   	.clkin(clk), 
   	.reset(reset),
	   .force_error(force_error),
   	.data_p(count[15:0]),
   	.code_out_p(encout),
   	.code_in_p(decin),
   	.m(decout[15:0]),
   	.error(),
   	.clkout(clk2x),
   	.locked(locked)
   	);   
   
   initial begin
   // initial reset
    reset = 1'b1;
   # (4 * tclk);
    reset = 1'b0;
   end
   
   // encoder output wrapped to decoder input

   assign decin = encout ;

   //----------------------------
   // clock driver
   //-----------------------------
   always 
   begin : clock_driver
      clk <= 1'b0 ; 
      #tclk; 
      forever
      begin
         clk <=  ~clk ;
         # (tclk / 2); 
      end 
   end 
     
   //  Test Bench Statements
   // main counter
   always @(clk2x or locked)
   begin : counter
      if (locked == 1'b0)
      begin
         count <= 32'h00000000 ; 
      end
      //			count <= 0;
      else if (clk2x == 1'b1)
      begin
         count <= count + 1 ; 
      end 
   end 

   //--------------------------------------
   // data generator
   always @(clk2x or reset or count)
   begin
      if (reset == 1'b1)
      begin
         force_error <= 2'b00 ; 
      end
      else if (clk2x == 1'b1)
      begin
         case (count)
            // feedthru
            32'h00000000 :
                     begin
                        force_error <= 3'b000 ; 
                        $display(" --- Data feedthru --- "); 
                     end
            // 1 bit error walk-thru
            32'h00001000 :
                     begin
                        force_error <= 3'b001 ; 
                        $display(" --- Starting single-bit error test --- "); 
                     end
            // 2-bit error walk-thru 				 		
            32'h00002000 :
                     begin
                        force_error <= 3'b010 ; 
                        $display(" --- Single-bit error test completed --- "); 
                        $display(" --- Starting double-bit error test --- "); 
                     end
            // 3-bit error walk-thru
            32'h00003000 :
                     begin
                        force_error <= 3'b011 ; 
                        $display(" --- Double-bit error test completed --- "); 
                        $display(" --- Starting triple-bit error test ---  "); 
                     end
            32'h00004000 :
                     begin
                        force_error <= 3'b100 ; 
                        $display(" --- triple-bit error test completed --- "); 
                        $display(" --- Starting quaduple-bit error test ---  "); 
                     end
            32'h00005000 :
                     begin
                        force_error <= 3'b101 ; 
                        $display(" --- quad-bit error test completed --- "); 
                        $display(" --- Starting pentuple-bit error test ---  "); 
                     end
            default :
                     begin
                        force_error <= force_error ; 
                     end
         endcase 
      end 
   end 

   // Verify the original data against the output of the decoder
   always @(clk2x or reset)
   begin : error_check
      if (reset == 1'b1)
      begin
         encout_1 <= 32'b0 ; 
         encout_2 <= 32'b0 ; 
         encout_3 <= 32'b0 ; 
         encout_4 <= 32'b0 ; 
         encout_5 <= 32'b0 ; 
         encout_6 <= 32'b0 ; 
         encout_7 <= 32'b0 ; 
      end
      else if (clk2x == 1'b0)
      begin
         encout_1 <= count ; 
         encout_2 <= encout_1 ; 
         encout_3 <= encout_2 ; 
         encout_4 <= encout_3 ; 
	    encout_5 <= encout_4 ; 
	    encout_6 <= encout_5 ; 
	    encout_7 <= encout_6 ;
	    encout_8 <= encout_7 ;
	    encout_9 <= encout_8 ;
         if (force_error[2] != 1'b1)
         begin
            if (decout != encout_9[15:0]) 
		begin
			$display("== data comparision error! =="); 
			$display("Time:",$time,"   input:", encout_9,"   output:",decout);
		end
         end 
      end 
   end 
   //  End Test Bench 
endmodule