//-----------------------------------------------------------------------------
// MLD_s1 - Module
//-----------------------------------------------------------------------------
//**************************************************************************
// 
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION \"AS IS\"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
//
//**************************************************************************
//-----------------------------------------------------------------------------
// Filename:        MLD_s1.v
//-----------------------------------------------------------------------------
// Description:
//      Top level design for 16-bit Multiple Error Detection and Correction.
//		  Further details can be found in XAPP715
//-----------------------------------------------------------------------------
//Structure:
//      --XAPP715_top.v
//        |--code_gen_s2.v
//        |--code_gen_s3.v
//        |--MLD_s1.v
//           |--MLD_adder_s1.v
//        |--MLD_s2.v
//           |--MLD_adder_s2.v
//        |--MLD_s3.v
//        |--ocs_gen_s1.v
//        |--ocs_gen_s2.v
//        |--encoder.v
//
//-----------------------------------------------------------------------------
// Owners:          Simon Tam 
// Revision 1.0
//
//     Modification History:                                                   
//     Date     Init          Description                                
//   --------  ------ ---------------------------------------------------------    
//   11/9/2004    STAM	    Initial Release 
//-----------------------------------------------------------------------------
`timescale 1ns/100ps
module MLD_s1(clk, reset, x9_s1,x8_s1,x7_s1,x6_s1,x5_s1,x4_s1,x3_s1,x2_s1,x1_s1,x0_s1,
                 x, error);
    input clk;
    input reset;
    input [7:0] x9_s1;
    input [7:0] x8_s1;
    input [7:0] x7_s1;
    input [7:0] x6_s1;
    input [7:0] x5_s1;
    input [7:0] x4_s1;
    input [7:0] x3_s1;
    input [7:0] x2_s1;
    input [7:0] x1_s1;
    input [7:0] x0_s1;
    output [9:0] x;
    output [1:0] error;

	wire [7:0] xp [9:0];
	wire [9:0] error_p;
	wire [9:0] y;
	wire  [9:0] x;
	reg  [1:0] error;
	wire [3:0] syndrome;

always @(posedge clk or posedge reset) begin
	if (reset) 
		begin
		error <= 2'b0;
		end
	else
		begin
			case (syndrome)
				4'b0000 : error <= 2'b00; // all zeros no error
				4'b1000 : error <= 2'b00; // all ones no error
				4'b0001 : error <= 2'b01;
				4'b0010 : error <= 2'b01;
				4'b0011 : error <= 2'b01;
				4'b0101 : error <= 2'b01;
				4'b0110 : error <= 2'b01;
				4'b0111 : error <= 2'b01;
				4'b0100 : error <= 2'b10; // four ones/zeros quad error	
				default : error <= 2'b11; // invalid errors
			endcase
		end
end


assign syndrome = x0_s1[7] + x0_s1[6] + x0_s1[5] + x0_s1[4] + x0_s1[3] + x0_s1[2] + x0_s1[1] + x0_s1[0];
assign x = y;
assign xp[9] = x9_s1;
assign xp[8] = x8_s1;
assign xp[7] = x7_s1;
assign xp[6] = x6_s1;
assign xp[5] = x5_s1;
assign xp[4] = x4_s1;
assign xp[3] = x3_s1;
assign xp[2] = x2_s1;
assign xp[1] = x1_s1;
assign xp[0] = x0_s1;


// replicate MLD for each bit using logic implementation
genvar i;
   generate
       for (i=0; i < 10; i=i+1) 
       begin: u1_MLD
		 MLD_adder_s1 inst (
		.datain(xp[i]),
		.dataout(y[i])
		);
       end
endgenerate

endmodule
