//-----------------------------------------------------------------------------
// MLD_s3 - Module
//-----------------------------------------------------------------------------
//**************************************************************************
// 
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION \"AS IS\"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
//
//**************************************************************************
//-----------------------------------------------------------------------------
// Filename:        MLD_s3.v
//-----------------------------------------------------------------------------
// Description:
//      Top level design for 16-bit Multiple Error Detection and Correction.
//		  Further details can be found in XAPP715
//-----------------------------------------------------------------------------
//Structure:
//      --XAPP715_top.v
//        |--code_gen_s2.v
//        |--code_gen_s3.v
//        |--MLD_s1.v
//           |--MLD_adder_s1.v
//        |--MLD_s2.v
//           |--MLD_adder_s2.v
//        |--MLD_s3.v
//        |--ocs_gen_s1.v
//        |--ocs_gen_s2.v
//        |--encoder.v
//
//-----------------------------------------------------------------------------
// Owners:          Simon Tam 
// Revision 1.0
//
//     Modification History:                                                   
//     Date     Init          Description                                
//   --------  ------ ---------------------------------------------------------    
//   11/9/2004    STAM	    Initial Release 
//-----------------------------------------------------------------------------
`timescale 1ns/100ps
module MLD_s3(clk,reset,x15_s3,x);
    input clk;
    input reset;
    input [31:0] x15_s3;
    output x;

wire [2:0] sum0, sum0_1, sum0_2, sum0_3, sum0_4, sum0_5, sum0_6, sum0_7;
wire [3:0] sum1_0, sum1_1, sum1_2, sum1_3;
wire [11:0] sum1;
wire [4:0] sum2_0, sum2_1;

wire [5:0] sum;

//reg   x;

assign sum0_0 = x15_s3[31] + x15_s3[30] + x15_s3[29] + x15_s3[28];
assign sum0_1 = x15_s3[27] + x15_s3[26] + x15_s3[25] + x15_s3[24];
assign sum0_2 = x15_s3[23] + x15_s3[22] + x15_s3[21] + x15_s3[20];
assign sum0_3 = x15_s3[19] + x15_s3[18] + x15_s3[17] + x15_s3[16];
assign sum0_4 = x15_s3[15] + x15_s3[14] + x15_s3[13] + x15_s3[12];
assign sum0_5 = x15_s3[11] + x15_s3[10] + x15_s3[9] + x15_s3[8];
assign sum0_6 = x15_s3[7] + x15_s3[6] + x15_s3[5] + x15_s3[4];
assign sum0_7 = x15_s3[3] + x15_s3[2] + x15_s3[1] + x15_s3[0];

/*
//assign sum0 = {sum0_7, sum0_6, sum0_5, sum0_4, sum0_3, sum0_2, sum0_1, sum0_0};
assign sum1 = {sum1_3, sum1_2, sum1_1, sum1_0};

always @ (sum1)
case (sum1)

16'bxxx1xxxxxxxx1000 : x <= 1'b1;
16'bxxxxxxx1xxxx1000 : x <= 1'b1;
16'bxxxxxxxxxxx11000 : x <= 1'b1;

16'bxxx1xxxx1000xxxx : x <= 1'b1;
16'bxxxxxxx11000xxxx : x <= 1'b1;
16'bxxxxxxxx1000xxx1 : x <= 1'b1;

16'bxxx11000xxxxxxxx : x <= 1'b1;
16'bxxxx1000xxx1xxxx : x <= 1'b1;
16'bxxxx1000xxxxxxx1 : x <= 1'b1;

16'b1000xxx1xxxxxxxx : x <= 1'b1;
16'b1000xxxxxxx1xxxx : x <= 1'b1;
16'b1000xxxxxxxxxxx1 : x <= 1'b1;

default : x <= 1'b0;
endcase
*/

assign sum1_0 = sum0_0 + sum0_1;
assign sum1_1 = sum0_2 + sum0_3;
assign sum1_2 = sum0_4 + sum0_5;
assign sum1_3 = sum0_6 + sum0_7;

assign sum2_0 = sum1_0 + sum1_1;
assign sum2_1 = sum1_2 + sum1_3;

assign sum= sum2_0 + sum2_1;
  

/*
assign sum0 = x15_s3[31] + x15_s3[30] + x15_s3[29] + x15_s3[28] 
+ x15_s3[27] + x15_s3[26] + x15_s3[25] + x15_s3[24];
assign sum1 = x15_s3[23] + x15_s3[22] + x15_s3[21] + x15_s3[20]
+ x15_s3[19] + x15_s3[18] + x15_s3[17] + x15_s3[16];
assign sum2 = x15_s3[15] + x15_s3[14] + x15_s3[13] + x15_s3[12]
+ x15_s3[11] + x15_s3[10] + x15_s3[9] + x15_s3[8];
assign sum3 = x15_s3[7] + x15_s3[6] + x15_s3[5] + x15_s3[4]
+ x15_s3[3] + x15_s3[2] + x15_s3[1] + x15_s3[0];
    
assign sum = sum0 + sum1 + sum2 + sum3;
    */
/*
assign sum = x15_s3[31] + x15_s3[30] + x15_s3[29] + x15_s3[28] 
+ x15_s3[27] + x15_s3[26] + x15_s3[25] + x15_s3[24]
+ x15_s3[23] + x15_s3[22] + x15_s3[21] + x15_s3[20]
+ x15_s3[19] + x15_s3[18] + x15_s3[17] + x15_s3[16]
+ x15_s3[15] + x15_s3[14] + x15_s3[13] + x15_s3[12]
+ x15_s3[11] + x15_s3[10] + x15_s3[9] + x15_s3[8]
+ x15_s3[7] + x15_s3[6] + x15_s3[5] + x15_s3[4]
+ x15_s3[3] + x15_s3[2] + x15_s3[1] + x15_s3[0];
*/

assign x = sum[4] ^ sum[5];
//assign x = sum[4];
endmodule
