//-----------------------------------------------------------------------------
// encoder - Module
//-----------------------------------------------------------------------------
//**************************************************************************
// 
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION \"AS IS\"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
//
//**************************************************************************
//-----------------------------------------------------------------------------
// Filename:        encoder.v
//-----------------------------------------------------------------------------
// Description:
//      Top level design for 16-bit Multiple Error Detection and Correction.
//		  Further details can be found in XAPP715
//-----------------------------------------------------------------------------
//Structure:
//      --XAPP715_top.v
//        |--code_gen_s2.v
//        |--code_gen_s3.v
//        |--MLD_s1.v
//           |--MLD_adder_s1.v
//        |--MLD_s2.v
//           |--MLD_adder_s2.v
//        |--MLD_s3.v
//        |--ocs_gen_s1.v
//        |--ocs_gen_s2.v
//        |--encoder.v
//
//-----------------------------------------------------------------------------
// Owners:          Simon Tam 
// Revision 1.0
//
//     Modification History:                                                   
//     Date     Init          Description                                
//   --------  ------ ---------------------------------------------------------    
//   11/9/2004    STAM	    Initial Release 
//-----------------------------------------------------------------------------
`timescale 1ns/100ps
module encoder(clk, reset, force_error, data,code_out);
    input clk;
    input reset;
    input [2:0]force_error;
    input [15:0] data;
    output [31:0] code_out;

   reg[31:0] single_error; 
   reg[31:0] double_error; 
   reg[31:0] triple_error;
   reg[31:0] quad_error;
   reg[31:0] pent_error;
   reg [31:0] code_out;
   wire [31:0] enc_in;


assign enc_in[0] = data[15] ^ data[14] ^ data[13] ^ data[12] ^ data[11] ^ data[10] ^ data[9] ^ data[8] ^ data[7] ^ data[6] ^ data[5] ^ data[4] ^ data[3] ^ data[2] ^ data[1] ^ data[0];
assign enc_in[1] = data[15] ^ data[13] ^ data[12] ^ data[11] ^ data[10] ^ data[5] ^ data[4] ^ data[3] ^ data[2] ^ data[1] ^ data[0];
assign enc_in[2] = data[15] ^ data[14] ^ data[12] ^ data[11] ^ data[10] ^ data[8] ^ data[7] ^ data[6] ^ data[2] ^ data[1] ^ data[0];
assign enc_in[3] = data[15] ^ data[12] ^ data[11] ^ data[10] ^ data[2] ^ data[1] ^ data[0];
assign enc_in[4] = data[15] ^ data[14] ^ data[13] ^ data[11] ^ data[10] ^ data[9] ^ data[7] ^ data[6] ^ data[4] ^ data[3] ^ data[0];
assign enc_in[5] = data[15] ^ data[13] ^ data[11] ^ data[10] ^ data[4] ^ data[3] ^ data[0];
assign enc_in[6] = data[15] ^ data[14] ^ data[11] ^ data[10] ^ data[7] ^ data[6] ^ data[0];
assign enc_in[7] = data[15] ^ data[11] ^ data[10] ^ data[0];
assign enc_in[8] = data[15] ^ data[14] ^ data[13] ^ data[12]^ data[10] ^ data[9] ^ data[8] ^ data[6] ^  data[5] ^ data[3] ^ data[1];
assign enc_in[9] = data[15] ^ data[13] ^ data[12] ^ data[10] ^  data[5] ^ data[3] ^ data[1];
assign enc_in[10] = data[15] ^ data[14] ^ data[12] ^ data[10] ^  data[8] ^ data[6] ^ data[1];
assign enc_in[11] = data[15] ^ data[12] ^ data[10] ^ data[1];
assign enc_in[12] = data[15] ^ data[14] ^ data[13] ^ data[10] ^  data[9] ^ data[6] ^ data[3];
assign enc_in[13] = data[15] ^ data[13] ^ data[10] ^ data[3];
assign enc_in[14] = data[15] ^ data[14] ^ data[10] ^ data[6];
assign enc_in[15] = data[15] ^ data[10];
assign enc_in[16] = data[15] ^ data[14] ^ data[13] ^ data[12] ^ data[11] ^ data[9] ^ data[8] ^ data[7] ^  data[5] ^ data[4] ^ data[2];
assign enc_in[17] = data[15] ^ data[13] ^ data[12] ^ data[11] ^  data[5] ^ data[4] ^ data[2];
assign enc_in[18] = data[15] ^ data[14] ^ data[12] ^ data[11] ^ data[8] ^ data[7] ^ data[2];
assign enc_in[19] = data[15] ^ data[12] ^ data[11] ^ data[2];
assign enc_in[20] = data[15] ^ data[14] ^ data[13] ^ data[11] ^ data[9] ^ data[7] ^ data[4];
assign enc_in[21] = data[15] ^ data[13] ^ data[11] ^ data[4];
assign enc_in[22] = data[15] ^ data[14] ^ data[11] ^ data[7];
assign enc_in[23] = data[15] ^ data[11];
assign enc_in[24] = data[15] ^ data[14] ^ data[13] ^ data[12] ^ data[9] ^ data[8] ^ data[5];
assign enc_in[25] = data[15] ^ data[13] ^ data[12] ^ data[5];
assign enc_in[26] = data[15] ^ data[14] ^ data[12] ^ data[8];
assign enc_in[27] = data[15] ^ data[12];
assign enc_in[28] = data[15] ^ data[14] ^ data[13] ^ data[9];
assign enc_in[29] = data[15] ^ data[13];
assign enc_in[30] = data[15] ^ data[14];
assign enc_in[31] = data[15];

always @(posedge clk or posedge reset)
   begin : single_error_gen
      if (reset == 1'b1)
      begin
         single_error <= 32'h00000001 ; 
      end
      else
      begin
         single_error[0] <= single_error[31] ; 
         single_error[31:1] <= single_error[30:0] ; 
      end 
   end 

   always @(posedge clk or posedge reset)
   begin : double_error_gen
      if (reset == 1'b1)
      begin
         double_error <= 32'h00000003 ; 
      end
      else
      begin
         double_error[0] <= double_error[31] ; 
         double_error[31:1] <= double_error[30:0] ; 
      end 
   end 

   always @(posedge clk or posedge reset)
   begin : triple_error_gen
      if (reset == 1'b1)
      begin
         triple_error <= 32'h00000007 ; 
      end
      else
      begin
         triple_error[0] <= triple_error[31] ; 
         triple_error[31:1] <= triple_error[30:0] ; 
      end 
   end 

   always @(posedge clk or posedge reset)
   begin : quad_error_gen
      if (reset == 1'b1)
      begin
         quad_error <= 32'h0000000f ; 
      end
      else
      begin
         quad_error[0] <= quad_error[31] ; 
         quad_error[31:1] <= quad_error[30:0] ; 
      end 
   end 

  always @(posedge clk or posedge reset)
   begin : pent_error_gen
      if (reset == 1'b1)
      begin
         pent_error <= 32'h0000001f ; 
      end
      else
      begin
         pent_error[0] <= pent_error[31] ; 
         pent_error[31:1] <= pent_error[30:0] ; 
      end 
   end 
always @(enc_in or force_error or single_error or double_error or triple_error or quad_error or pent_error)
 
         case (force_error)
            3'b000 :	 code_out <= enc_in ^ 32'h00000000 ;  // 0 error feedthrough     
            3'b001 :	 code_out <= enc_in ^ single_error[31:0] ; 	// 1 bit error walk-thru
            3'b010 :	 code_out <= enc_in ^ double_error[31:0] ;	// 2-bit error walk-thru
            3'b011 :	 code_out <= enc_in ^ triple_error[31:0] ;  
            3'b100 :	 code_out <= enc_in ^ quad_error[31:0] ;  
            3'b101 :	 code_out <= enc_in ^ pent_error[31:0] ; 
            default :  code_out <= enc_in ;           
         endcase 


endmodule

// The encoder implements the following RM[2,5] matrix

//		1	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1
//X1		0	1	0	1	0	1	0	1	0	1	0	1	0	1	0	1	0	1	0	1	0	1	0	1	0	1	0	1	0	1	0	1
//X2		0	0	1	1	0	0	1	1	0	0	1	1	0	0	1	1	0	0	1	1	0	0	1	1	0	0	1	1	0	0	1	1
//X3		0	0	0	0	1	1	1	1	0	0	0	0	1	1	1	1	0	0	0	0	1	1	1	1	0	0	0	0	1	1	1	1
//X4		0	0	0	0	0	0	0	0	1	1	1	1	1	1	1	1	0	0	0	0	0	0	0	0	1	1	1	1	1	1	1	1
//X5		0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1
//X1X2	0	0	0	1	0	0	0	1	0	0	0	1	0	0	0	1	0	0	0	1	0	0	0	1	0	0	0	1	0	0	0	1
//X1X3	0	0	0	0	0	1	0	1	0	0	0	0	0	1	0	1	0	0	0	0	0	1	0	1	0	0	0	0	0	1	0	1
//X1X4	0	0	0	0	0	0	0	0	0	1	0	1	0	1	0	1	0	0	0	0	0	0	0	0	0	1	0	1	0	1	0	1
//X1X5	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	1	0	1	0	1	0	1	0	1	0	1	0	1
//X2X3	0	0	0	0	0	0	1	1	0	0	0	0	0	0	1	1	0	0	0	0	0	0	1	1	0	0	0	0	0	0	1	1
//X2X4	0	0	0	0	0	0	0	0	0	0	1	1	0	0	1	1	0	0	0	0	0	0	0	0	0	0	1	1	0	0	1	1
//X2X5	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	1	0	0	1	1	0	0	1	1	0	0	1	1
//X3X4	0	0	0	0	0	0	0	0	0	0	0	0	1	1	1	1	0	0	0	0	0	0	0	0	0	0	0	0	1	1	1	1
//X3X5	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	1	1	1	0	0	0	0	1	1	1	1
//X4X5	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	1	1	1	1	1	1	1	1

