//-----------------------------------------------------------------------------
// ocs_gen_s1 - Module
//-----------------------------------------------------------------------------
//**************************************************************************
// 
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION \"AS IS\"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
//
//**************************************************************************
//-----------------------------------------------------------------------------
// Filename:        ocs_gen_s1.v
//-----------------------------------------------------------------------------
// Description:
//      Top level design for 16-bit Multiple Error Detection and Correction.
//		  Further details can be found in XAPP715
//-----------------------------------------------------------------------------
//Structure:
//      --XAPP715_top.v
//        |--code_gen_s2.v
//        |--code_gen_s3.v
//        |--MLD_s1.v
//           |--MLD_adder_s1.v
//        |--MLD_s2.v
//           |--MLD_adder_s2.v
//        |--MLD_s3.v
//        |--ocs_gen_s1.v
//        |--ocs_gen_s2.v
//        |--encoder.v
//
//-----------------------------------------------------------------------------
// Owners:          Simon Tam 
// Revision 1.0
//
//     Modification History:                                                   
//     Date     Init          Description                                
//   --------  ------ ---------------------------------------------------------    
//   11/9/2004    STAM	    Initial Release 
//-----------------------------------------------------------------------------
`timescale 1ns/100ps
module ocs_gen_s1(code_in,x9_s1,x8_s1,x7_s1,x6_s1,x5_s1,x4_s1,x3_s1,x2_s1,
                     x1_s1,x0_s1);
    input [31:0] code_in;
    output [7:0] x9_s1;
    output [7:0] x8_s1;
    output [7:0] x7_s1;
    output [7:0] x6_s1;
    output [7:0] x5_s1;
    output [7:0] x4_s1;
    output [7:0] x3_s1;
    output [7:0] x2_s1;
    output [7:0] x1_s1;
    output [7:0] x0_s1;

assign x9_s1[7] = code_in[0] ^ code_in[1] ^ code_in[2] ^ code_in[3];
assign x9_s1[6] = code_in[4] ^ code_in[5] ^ code_in[6] ^ code_in[7];
assign x9_s1[5] = code_in[8] ^ code_in[9] ^ code_in[10] ^ code_in[11];
assign x9_s1[4] = code_in[12] ^ code_in[13] ^ code_in[14] ^ code_in[15];
assign x9_s1[3] = code_in[16] ^ code_in[17] ^ code_in[18] ^ code_in[19];
assign x9_s1[2] = code_in[20] ^ code_in[21] ^ code_in[22] ^ code_in[23];
assign x9_s1[1] = code_in[24] ^ code_in[25] ^ code_in[26] ^ code_in[27];
assign x9_s1[0] = code_in[28] ^ code_in[29] ^ code_in[30] ^ code_in[31];

assign x8_s1[7] = code_in[0] ^ code_in[1] ^ code_in[4] ^ code_in[5];
assign x8_s1[6] = code_in[2] ^ code_in[3] ^ code_in[6] ^ code_in[7];
assign x8_s1[5] = code_in[8] ^ code_in[9] ^ code_in[12] ^ code_in[13];
assign x8_s1[4] = code_in[10] ^ code_in[11] ^ code_in[14] ^ code_in[15];
assign x8_s1[3] = code_in[16] ^ code_in[17] ^ code_in[20] ^ code_in[21];
assign x8_s1[2] = code_in[18] ^ code_in[19] ^ code_in[22] ^ code_in[23];
assign x8_s1[1] = code_in[24] ^ code_in[25] ^ code_in[28] ^ code_in[29];
assign x8_s1[0] = code_in[26] ^ code_in[27] ^ code_in[30] ^ code_in[31];

assign x7_s1[7] = code_in[9] ^ code_in[8] ^ code_in[1] ^ code_in[0];
assign x7_s1[6] = code_in[11] ^ code_in[10] ^ code_in[3] ^ code_in[2];
assign x7_s1[5] = code_in[13] ^ code_in[12] ^ code_in[5] ^ code_in[4];
assign x7_s1[4] = code_in[25] ^ code_in[24] ^ code_in[17] ^ code_in[16];
assign x7_s1[3] = code_in[15] ^ code_in[14] ^ code_in[7] ^ code_in[6];
assign x7_s1[2] = code_in[27] ^ code_in[26] ^ code_in[19] ^ code_in[18];
assign x7_s1[1] = code_in[29] ^ code_in[28] ^ code_in[21] ^ code_in[20];
assign x7_s1[0] = code_in[31] ^ code_in[30] ^ code_in[23] ^ code_in[22];

assign x6_s1[7] = code_in[17] ^ code_in[16] ^ code_in[1] ^ code_in[0];
assign x6_s1[6] = code_in[19] ^ code_in[18] ^ code_in[3] ^ code_in[2];
assign x6_s1[5] = code_in[21] ^ code_in[20] ^ code_in[5] ^ code_in[4];
assign x6_s1[4] = code_in[25] ^ code_in[24] ^ code_in[9] ^ code_in[8];
assign x6_s1[3] = code_in[23] ^ code_in[22] ^ code_in[7] ^ code_in[6];
assign x6_s1[2] = code_in[27] ^ code_in[26] ^ code_in[11] ^ code_in[10];
assign x6_s1[1] = code_in[29] ^ code_in[28] ^ code_in[13] ^ code_in[12];
assign x6_s1[0] = code_in[31] ^ code_in[30] ^ code_in[15] ^ code_in[14];

assign x5_s1[7] = code_in[6] ^ code_in[4]^ code_in[2] ^ code_in[0];
assign x5_s1[6] = code_in[7] ^ code_in[5]^ code_in[3] ^ code_in[1];
assign x5_s1[5] = code_in[14] ^ code_in[12]^ code_in[10] ^ code_in[8];
assign x5_s1[4] = code_in[22] ^ code_in[20]^ code_in[18] ^ code_in[16];
assign x5_s1[3] = code_in[15] ^ code_in[13]^ code_in[11] ^ code_in[9];
assign x5_s1[2] = code_in[23] ^ code_in[21]^ code_in[19] ^ code_in[17];
assign x5_s1[1] = code_in[30] ^ code_in[28]^ code_in[26] ^ code_in[24];
assign x5_s1[0] = code_in[31] ^ code_in[29]^ code_in[27] ^ code_in[25];

assign x4_s1[7] = code_in[10] ^ code_in[8]^ code_in[2]^ code_in[0];
assign x4_s1[6] =  code_in[11] ^ code_in[9]^ code_in[3]^ code_in[1];
assign x4_s1[5] =  code_in[14] ^ code_in[12]^ code_in[6]^ code_in[4];
assign x4_s1[4] =  code_in[26] ^ code_in[24]^ code_in[18]^ code_in[16];
assign x4_s1[3] =  code_in[15] ^ code_in[13]^ code_in[7]^ code_in[5];
assign x4_s1[2] =  code_in[27] ^ code_in[25]^ code_in[19]^ code_in[17];
assign x4_s1[1] =  code_in[30] ^ code_in[28]^ code_in[22]^ code_in[20];
assign x4_s1[0] =  code_in[31] ^ code_in[29]^ code_in[23]^ code_in[21];

assign x3_s1[7] = code_in[18] ^ code_in[16] ^ code_in[2]^ code_in[0];
assign x3_s1[6] = code_in[19] ^ code_in[17] ^ code_in[3]^ code_in[1];
assign x3_s1[5] = code_in[22] ^ code_in[20] ^ code_in[6]^ code_in[4];
assign x3_s1[4] = code_in[26] ^ code_in[24] ^ code_in[10]^ code_in[8];
assign x3_s1[3] = code_in[23] ^ code_in[21] ^ code_in[7]^ code_in[5];
assign x3_s1[2] = code_in[27] ^ code_in[25] ^ code_in[11]^ code_in[9];
assign x3_s1[1] = code_in[30] ^ code_in[28] ^ code_in[14]^ code_in[12];
assign x3_s1[0] = code_in[31] ^ code_in[29] ^ code_in[15]^ code_in[13];

assign x2_s1[7] = code_in[12]^ code_in[8]^ code_in[4]^ code_in[0];
assign x2_s1[6] = code_in[13]^ code_in[9]^ code_in[5]^ code_in[1];
assign x2_s1[5] = code_in[14]^ code_in[10]^ code_in[6]^ code_in[2];
assign x2_s1[4] = code_in[28]^ code_in[24]^ code_in[20]^ code_in[16];
assign x2_s1[3] = code_in[15]^ code_in[11]^ code_in[7]^ code_in[3];
assign x2_s1[2] = code_in[29]^ code_in[25]^ code_in[21]^ code_in[17];
assign x2_s1[1] = code_in[30]^ code_in[26]^ code_in[22]^ code_in[18];
assign x2_s1[0] = code_in[31]^ code_in[27]^ code_in[23]^ code_in[19];

assign x1_s1[7] = code_in[20]^ code_in[16]^ code_in[4]^ code_in[0];
assign x1_s1[6] = code_in[21]^ code_in[17]^ code_in[5]^ code_in[1];
assign x1_s1[5] = code_in[22]^ code_in[18]^ code_in[6]^ code_in[2];
assign x1_s1[4] = code_in[28]^ code_in[24]^ code_in[12]^ code_in[8];
assign x1_s1[3] = code_in[23]^ code_in[19]^ code_in[7]^ code_in[3];
assign x1_s1[2] = code_in[29]^ code_in[25]^ code_in[13]^ code_in[9];
assign x1_s1[1] = code_in[30]^ code_in[26]^ code_in[14]^ code_in[10];
assign x1_s1[0] = code_in[31]^ code_in[27]^ code_in[15]^ code_in[11];

assign x0_s1[7] = code_in[24]^ code_in[16]^ code_in[8]^ code_in[0];
assign x0_s1[6] = code_in[25]^ code_in[17]^ code_in[9]^ code_in[1];
assign x0_s1[5] = code_in[26]^ code_in[18]^ code_in[10]^ code_in[2];
assign x0_s1[4] = code_in[28]^ code_in[20]^ code_in[12]^ code_in[4];
assign x0_s1[3] = code_in[27]^ code_in[19]^ code_in[11]^ code_in[3];
assign x0_s1[2] = code_in[29]^ code_in[21]^ code_in[13]^ code_in[5];
assign x0_s1[1] = code_in[30]^ code_in[22]^ code_in[14]^ code_in[6];
assign x0_s1[0] = code_in[31]^ code_in[23]^ code_in[15]^ code_in[7];


endmodule
