//-----------------------------------------------------------------------------
// ocs_gen_s2 - Module
//-----------------------------------------------------------------------------
//**************************************************************************
// 
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION \"AS IS\"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
//
//**************************************************************************
//-----------------------------------------------------------------------------
// Filename:        ocs_gen_s2.v
//-----------------------------------------------------------------------------
// Description:
//      Top level design for 16-bit Multiple Error Detection and Correction.
//		  Further details can be found in XAPP715
//-----------------------------------------------------------------------------
//Structure:
//      --XAPP715_top.v
//        |--code_gen_s2.v
//        |--code_gen_s3.v
//        |--MLD_s1.v
//           |--MLD_adder_s1.v
//        |--MLD_s2.v
//           |--MLD_adder_s2.v
//        |--MLD_s3.v
//        |--ocs_gen_s1.v
//        |--ocs_gen_s2.v
//        |--encoder.v
//
//-----------------------------------------------------------------------------
// Owners:          Simon Tam 
// Revision 1.0
//
//     Modification History:                                                   
//     Date     Init          Description                                
//   --------  ------ ---------------------------------------------------------    
//   11/9/2004    STAM	    Initial Release 
//-----------------------------------------------------------------------------
`timescale 1ns/100ps
module ocs_gen_s2(code_in,x14_s2,x13_s2,x12_s2,x11_s2,x10_s2);
    input [31:0] code_in;
    output [15:0] x14_s2;
    output [15:0] x13_s2;
    output [15:0] x12_s2;
    output [15:0] x11_s2;
    output [15:0] x10_s2;

assign x14_s2[15] = code_in[1]^ code_in[0];
assign x14_s2[14] = code_in[17]^ code_in[16];
assign x14_s2[13] = code_in[9]^ code_in[8];
assign x14_s2[12] = code_in[25]^ code_in[24];
assign x14_s2[11] = code_in[5]^ code_in[4];
assign x14_s2[10] = code_in[21]^ code_in[20];
assign x14_s2[9] = code_in[13]^ code_in[12];
assign x14_s2[8] = code_in[29]^ code_in[28];
assign x14_s2[7] = code_in[3]^ code_in[2];
assign x14_s2[6] = code_in[19]^ code_in[18];
assign x14_s2[5] = code_in[11]^ code_in[10];
assign x14_s2[4] = code_in[27]^ code_in[26];
assign x14_s2[3] = code_in[7]^ code_in[6];
assign x14_s2[2] = code_in[23]^ code_in[22];
assign x14_s2[1] = code_in[15]^ code_in[14];
assign x14_s2[0] = code_in[31]^ code_in[30];

assign x13_s2[15] = code_in[2]^ code_in[0];
assign x13_s2[14] = code_in[18]^ code_in[16];
assign x13_s2[13] = code_in[10]^ code_in[8];
assign x13_s2[12] = code_in[26]^ code_in[24];
assign x13_s2[11] = code_in[6]^ code_in[4];
assign x13_s2[10] = code_in[22]^ code_in[20];
assign x13_s2[9] = code_in[14]^ code_in[12];
assign x13_s2[8] = code_in[30]^ code_in[28];
assign x13_s2[7] = code_in[3]^ code_in[1];
assign x13_s2[6] = code_in[19]^ code_in[17];
assign x13_s2[5] = code_in[11]^ code_in[9];
assign x13_s2[4] = code_in[27]^ code_in[25];
assign x13_s2[3] = code_in[7]^ code_in[5];
assign x13_s2[2] = code_in[23]^ code_in[21];
assign x13_s2[1] = code_in[15]^ code_in[13];
assign x13_s2[0] = code_in[31]^ code_in[29];

assign x12_s2[15] = code_in[4]^ code_in[0];
assign x12_s2[14] = code_in[20]^ code_in[16];
assign x12_s2[13] = code_in[12]^ code_in[8];
assign x12_s2[12] = code_in[28]^ code_in[24];
assign x12_s2[11] = code_in[6]^ code_in[2];
assign x12_s2[10] = code_in[22]^ code_in[18];
assign x12_s2[9] = code_in[14]^ code_in[10];
assign x12_s2[8] = code_in[30]^ code_in[26];
assign x12_s2[7] = code_in[5]^ code_in[1];
assign x12_s2[6] = code_in[21]^ code_in[17];
assign x12_s2[5] = code_in[13]^ code_in[9];
assign x12_s2[4] = code_in[29]^ code_in[25];
assign x12_s2[3] = code_in[7]^ code_in[3];
assign x12_s2[2] = code_in[23]^ code_in[19];
assign x12_s2[1] = code_in[15]^ code_in[11];
assign x12_s2[0] = code_in[31]^ code_in[27];

assign x11_s2[15] = code_in[8]^ code_in[0];
assign x11_s2[14] = code_in[24]^ code_in[16];
assign x11_s2[13] = code_in[12]^ code_in[4];
assign x11_s2[12] = code_in[28]^ code_in[20];
assign x11_s2[11] = code_in[10]^ code_in[2];
assign x11_s2[10] = code_in[26]^ code_in[18];
assign x11_s2[9] = code_in[14]^ code_in[6];
assign x11_s2[8] = code_in[30]^ code_in[22];
assign x11_s2[7] = code_in[9]^ code_in[1];
assign x11_s2[6] = code_in[25]^ code_in[17];
assign x11_s2[5] = code_in[13]^ code_in[5];
assign x11_s2[4] = code_in[29]^ code_in[21];
assign x11_s2[3] = code_in[11]^ code_in[3];
assign x11_s2[2] = code_in[27]^ code_in[19];
assign x11_s2[1] = code_in[15]^ code_in[7];
assign x11_s2[0] = code_in[31]^ code_in[23];

assign x10_s2[15] = code_in[16]^ code_in[0];
assign x10_s2[14] = code_in[24]^ code_in[8];
assign x10_s2[13] = code_in[20]^ code_in[4] ;
assign x10_s2[12] = code_in[28]^ code_in[12];
assign x10_s2[11] = code_in[18]^ code_in[2] ;
assign x10_s2[10] = code_in[26]^ code_in[10];
assign x10_s2[9] = code_in[22]^ code_in[6];
assign x10_s2[8] = code_in[30]^ code_in[14];
assign x10_s2[7] = code_in[17]^ code_in[1];
assign x10_s2[6] = code_in[25]^ code_in[9];
assign x10_s2[5] = code_in[21]^ code_in[5];
assign x10_s2[4] = code_in[29]^ code_in[13];
assign x10_s2[3] = code_in[19]^ code_in[3];
assign x10_s2[2] = code_in[27]^ code_in[11];
assign x10_s2[1] = code_in[23]^ code_in[7];
assign x10_s2[0] = code_in[31]^ code_in[15];

endmodule
